import pygame
import pygame.locals as pgl

class Events(object):
    instance = None

    def __init__(self, mapping = None):
        self.actions = {
                'escape': False,
                'up': False,
                'down': False,
                'left': False,
                'right': False,
                'action1': False, #TODO: rename appropiately
                'action2': False,  #TODO: rename appropiately
                'action3': False,  #TODO: rename appropiately
                'run': False,
                'screenshot': False,
                }
        self.mouse = {
                "x": 0,
                "y": 0,
                'mouse1': False,
                'mouse2': False,
                }
        if mapping:
            self.mapping = mapping
        else:
            self.mapping = {
                    pgl.K_ESCAPE: 'escape',
                    pgl.K_UP: 'up',
                    pgl.K_DOWN: 'down',
                    pgl.K_LEFT: 'left',
                    pgl.K_RIGHT: 'right',
                    pgl.K_SPACE: 'action1',
                    pgl.K_LCTRL: 'action2',
                    pgl.K_LSHIFT: 'run',
                    pgl.K_LALT: 'action3',
                    pgl.K_p: 'screenshot',
                    }
        self.j_mapping = {
                0: 'action1',
                1: 'action2',
                2: 'run',
                }
        self.joystick_treshold = 0.1

        try:
            pygame.joystick.init()
            for j in xrange(pygame.joystick.get_count()):
                pygame.joystick.Joystick(j).init()
        except:
            print "no joystick support"


    def collect_events(self):
        for event in pygame.event.get():
            if event.type == pgl.KEYUP or event.type == pgl.KEYDOWN:
                self.key_event(event)
            if event.type == pgl.MOUSEBUTTONUP or event.type == pgl.MOUSEBUTTONDOWN:
                self.mouse_button_event(event)
            if event.type == pgl.MOUSEMOTION:
                self.mouse_motion_event(event)
            if event.type == pgl.JOYAXISMOTION:
                self.joystick_motion_event(event)
            if event.type == pgl.JOYBUTTONUP or event.type == pgl.JOYBUTTONDOWN:
                self.joystick_button_event(event)

    def joystick_button_event(self, event):
        state = event.type == pgl.JOYBUTTONDOWN
        if self.j_mapping.has_key(event.button):
            self.actions[self.j_mapping[event.button]] = state

    def joystick_motion_event(self, event):
        tr = self.joystick_treshold
        if event.axis == 0:
            if event.value > tr:
                self.actions["right"] = True
                self.actions["left"] = False
            elif event.value < -tr:
                self.actions["left"] = True
                self.actions["right"] = False
            else:
                self.actions["left"] = self.actions["right"] = False
        elif event.axis == 1:
            if event.value > tr:
                self.actions["down"] = True
                self.actions["up"] = False
            elif event.value < -tr:
                self.actions["up"] = True
                self.actions["down"] = False
            else:
                self.actions["up"] = self.actions["down"] = False

    def mouse_button_event(self, event):
        down = event.type == pgl.MOUSEBUTTONDOWN
        if event.button == 1:
            self.mouse["mouse1"] = down
        if event.button == 3:
            self.mouse["mouse2"] = down

    def mouse_motion_event(self, event):
        self.mouse["x"] = event.pos[0]
        self.mouse["y"] = event.pos[1]

    def key_event(self, event):
        key = event.key
        if self.mapping.has_key(key):
            self.actions[self.mapping[key]] = event.type == pgl.KEYDOWN

    def get(self, action):
        return self.actions[action]

    def clear(self):
        for action in self.actions:
            self.actions[action] = False

    def __getattr__(self, name):
        if name == "mouse":
            return self.mouse
        return self.actions[name]

    @staticmethod
    def get_instance():
        if not instance:
            raise RuntimeError("Instance requested before initialisation")

        return Events.instance

