import pygame
import pygame.locals as pgl
import states
import events
import data
import music
from random import choice
from text import Text

import sys

import time

class Game(object):

    instance = None

    def __init__(self, resolution, fps = 30):
        self.resolution = width, height = resolution
        self.screen = pygame.display.set_mode(self.resolution)
        self.fps = fps

        Game.instance = self
        self.clock = pygame.time.Clock()

        self.events = events.Events()
        self.surfaces = data.Surfaces()

        self.states = states.StateManager(self, self.resolution)
        pygame.display.set_caption("Dark Room")
        self.text = Text()
        pygame.mixer.init()
        self.sounds = {
                "attack": [
                    pygame.mixer.Sound(data.filepath(data.create_relpath("sounds", "swing1.ogg"))),
                    pygame.mixer.Sound(data.filepath(data.create_relpath("sounds", "swing2.ogg"))),
                    pygame.mixer.Sound(data.filepath(data.create_relpath("sounds", "swing3.ogg"))),
                    ],
                "spider": [
                    pygame.mixer.Sound(data.filepath(data.create_relpath("sounds", "bite-small1.ogg"))),
                    pygame.mixer.Sound(data.filepath(data.create_relpath("sounds", "bite-small2.ogg"))),
                    pygame.mixer.Sound(data.filepath(data.create_relpath("sounds", "bite-small3.ogg"))),
                    ],
                "hit": [
                    pygame.mixer.Sound(data.filepath(data.create_relpath("sounds", "hit.ogg"))),
                    ]
                }

    def play_sound(self, name):
        choice(self.sounds[name]).play()

    @staticmethod
    def get_instance():
        if not Game.instance:
            raise RuntimeError("Instance requested before initialisation")

        return Game.instance

    def quit(self):
        self.running = False

    def run(self):
        self.running = True
        self.music = music.Music()
        self.music.play()
        while self.running:
            for event in pygame.event.get(pgl.QUIT):
                self.quit()
                return
            for event in pygame.event.get(music.MUSIC_END_EVENT):
                self.music.handle_event(event)
            self.events.collect_events()

            if self.events.screenshot:
                fn = time.strftime("screenshot_%Y-%m-%d_%H-%M-%S-")
                fn += str(pygame.time.get_ticks()) + ".png"
                fn = data.create_relpath("screenshots", fn)
                filepath = data.filepath(fn)
                pygame.image.save(self.screen, filepath)

            self.states.handle_events(self.events)

            delta = self.clock.tick(self.fps)

            #print "\r", self.clock.get_fps(), 
            #sys.stdout.flush()

            self.states.update(delta)
            self.states.draw(self.screen)

        print "Thanks for playing!"
            
    def get_surfaces(self):
        return self.surfaces

    def get_resolution(self):
        return self.resolution

    def get_events(self):
        return self.events
