import pygame
import pygame.locals as pgl

import data

import os

from pytmx import util_pygame

class MapObject(object):
    def __init__(self, filename):
        self.path = os.path.join("maps", filename)
        self.tmxdata = util_pygame.load_pygame(data.filepath(self.path))
        self.surface = None

        tmxdata = self.tmxdata
        self.width = tmxdata.width
        self.height = tmxdata.height
        self.width = tmxdata.width
        self.tile_h = tmxdata.tileheight
        self.tile_w = tmxdata.tilewidth

        self.pixel_h = self.height * self.tile_h
        self.pixel_w = self.width * self.tile_w

    def get_dimensions(self):
        return (self.pixel_w, self.pixel_h)

    def get_dimensions_tile(self):
        return self.width, self.height

    def get_surface(self):
        if self.surface:
            return self.surface

        tmxdata = self.tmxdata
        height = self.height
        width = self.width
        tile_h = self.tile_h
        tile_w = self.tile_w

        sf = pygame.Surface((width * tile_w, height * tile_h))

        for x in xrange(width):
            for y in xrange(height):
                for l in tmxdata.visible_tile_layers:
                    image = tmxdata.get_tile_image(x, y, l)
                    if image:
                        sf.blit(image, (x * tile_w, y * tile_h))

        self.surface = sf
        return self.surface

    def get_walls(self):
        collision_layer = self.tmxdata.get_layer_by_name("collision")

        walls = []
        for wall in collision_layer:
            walls.append(pygame.Rect(wall.x, wall.y, wall.width, wall.height))

        return walls

    def get_player(self):
        player = self.tmxdata.get_object_by_name("player")

        return (player.x + player.width // 2, player.y + player.height // 2) 

    def get_object(self, name):
        obj = self.tmxdata.get_object_by_name(name)
        return (obj.x + obj.width // 2, obj.y + obj.height / 2)

