import pygame
import pygame.locals as pgl

from gameobjects import updateable

class Viewport(updateable.Updateable):

    instance = None

    ''' follow an object (e.g. player) with the "camera" while stopping on the level bounds'''
    def __init__(self, width, height, resolution, focus):
        self._width = width
        self._height = height

        self.resolution = resolution

        self.viewport = pygame.Rect((0, 0), self.resolution)

        # expect a pygame.Rect
        self.focus = focus

        self._focus()

        Viewport.instance = self

    @staticmethod
    def get_instance():
        return Viewport.instance

    def update(self, delta = 0):
        self._focus()

    def get_bounds(self):
        return self.viewport

    def __getattr__(self, name):
        return self.viewport.__getattribute__(name)

    def _focus(self):
        self.viewport.center = self.focus.center

        if self.viewport.left < 0:
            self.viewport.left = 0
        if self.viewport.right > self._width:
            self.viewport.right = self._width
        if self.viewport.top < 0:
            self.viewport.top = 0
        if self.viewport.bottom > self._height:
            self.viewport.bottom = self._height

    def change_focus(self, focus):
        self.focus = focus
        self._focus()


